@@ -1122,6 +1122,102 @@ size_t DecodeStruct(const uint8_t* buffer, size_t buffer_size, Decoded_D3D12_STA
             bytes_read += wrapper->raytracing_pipeline_config1->Decode(buffer2, buffer_size2);
             value->pDesc = wrapper->raytracing_pipeline_config1->GetPointer();
             break;
+        case D3D12_STATE_SUBOBJECT_TYPE_WORK_GRAPH:
+            wrapper->work_graph_desc =
+                DecodeAllocator::Allocate<StructPointerDecoder<Decoded_D3D12_WORK_GRAPH_DESC>>();
+            bytes_read += wrapper->work_graph_desc->Decode(buffer2, buffer_size2);
+            value->pDesc = wrapper->work_graph_desc->GetPointer();
+            break;
+        case D3D12_STATE_SUBOBJECT_TYPE_STREAM_OUTPUT:
+            wrapper->stream_output_desc =
+                DecodeAllocator::Allocate<StructPointerDecoder<Decoded_D3D12_STREAM_OUTPUT_DESC>>();
+            bytes_read += wrapper->stream_output_desc->Decode(buffer2, buffer_size2);
+            value->pDesc = wrapper->stream_output_desc->GetPointer();
+            break;
+        case D3D12_STATE_SUBOBJECT_TYPE_BLEND:
+            wrapper->blend_desc = DecodeAllocator::Allocate<StructPointerDecoder<Decoded_D3D12_BLEND_DESC>>();
+            bytes_read += wrapper->blend_desc->Decode(buffer2, buffer_size2);
+            value->pDesc = wrapper->blend_desc->GetPointer();
+            break;
+        case D3D12_STATE_SUBOBJECT_TYPE_SAMPLE_MASK:
+            bytes_read += ValueDecoder::DecodeUInt32Value((buffer + bytes_read),
+                                                          (buffer_size - bytes_read),
+                                                          reinterpret_cast<uint32_t*>(const_cast<void*>(value->pDesc)));
+            break;
+        case D3D12_STATE_SUBOBJECT_TYPE_RASTERIZER:
+            wrapper->rasterizer_desc2 = DecodeAllocator::Allocate<StructPointerDecoder<Decoded_D3D12_RASTERIZER_DESC2>>();
+            bytes_read += wrapper->rasterizer_desc2->Decode(buffer2, buffer_size2);
+            value->pDesc = wrapper->rasterizer_desc2->GetPointer();
+            break;
+        case D3D12_STATE_SUBOBJECT_TYPE_DEPTH_STENCIL:
+            wrapper->depth_stencil_desc = DecodeAllocator::Allocate<StructPointerDecoder<Decoded_D3D12_DEPTH_STENCIL_DESC>>();
+            bytes_read += wrapper->depth_stencil_desc->Decode(buffer2, buffer_size2);
+            value->pDesc = wrapper->depth_stencil_desc->GetPointer();
+            break;
+        case D3D12_STATE_SUBOBJECT_TYPE_INPUT_LAYOUT:
+            wrapper->input_layout_desc = DecodeAllocator::Allocate<StructPointerDecoder<Decoded_D3D12_INPUT_LAYOUT_DESC>>();
+            bytes_read += wrapper->input_layout_desc->Decode(buffer2, buffer_size2);
+            value->pDesc = wrapper->input_layout_desc->GetPointer();
+            break;
+        case D3D12_STATE_SUBOBJECT_TYPE_IB_STRIP_CUT_VALUE:
+            // enum D3D12_INDEX_BUFFER_STRIP_CUT_VALUE
+            bytes_read += ValueDecoder::DecodeEnumValue(
+                (buffer + bytes_read),
+                (buffer_size - bytes_read),
+                reinterpret_cast<D3D12_INDEX_BUFFER_STRIP_CUT_VALUE*>(const_cast<void*>(value->pDesc)));
+            break;
+        case D3D12_STATE_SUBOBJECT_TYPE_PRIMITIVE_TOPOLOGY:
+            // enum D3D12_PRIMITIVE_TOPOLOGY_TYPE
+            bytes_read += ValueDecoder::DecodeEnumValue(
+                (buffer + bytes_read),
+                (buffer_size - bytes_read),
+                reinterpret_cast<D3D12_PRIMITIVE_TOPOLOGY_TYPE*>(const_cast<void*>(value->pDesc)));
+            break;
+        case D3D12_STATE_SUBOBJECT_TYPE_RENDER_TARGET_FORMATS:
+            wrapper->rt_format_array =
+                DecodeAllocator::Allocate<StructPointerDecoder<Decoded_D3D12_RT_FORMAT_ARRAY>>();
+            bytes_read += wrapper->rt_format_array->Decode(buffer2, buffer_size2);
+            value->pDesc = wrapper->rt_format_array->GetPointer();
+            break;
+        case D3D12_STATE_SUBOBJECT_TYPE_DEPTH_STENCIL_FORMAT:
+            // enum DXGI_FORMAT
+            bytes_read +=
+                ValueDecoder::DecodeEnumValue((buffer + bytes_read),
+                                              (buffer_size - bytes_read),
+                                              reinterpret_cast<DXGI_FORMAT*>(const_cast<void*>(value->pDesc)));
+            break;
+        case D3D12_STATE_SUBOBJECT_TYPE_SAMPLE_DESC:
+            wrapper->dxgi_sample_desc = DecodeAllocator::Allocate<StructPointerDecoder<Decoded_DXGI_SAMPLE_DESC>>();
+            bytes_read += wrapper->dxgi_sample_desc->Decode(buffer2, buffer_size2);
+            value->pDesc = wrapper->dxgi_sample_desc->GetPointer();
+            break;
+        case D3D12_STATE_SUBOBJECT_TYPE_FLAGS:
+            // enum D3D12_PIPELINE_STATE_FLAGS
+            bytes_read += ValueDecoder::DecodeFlagsValue(
+                (buffer + bytes_read),
+                (buffer_size - bytes_read),
+                reinterpret_cast<D3D12_PIPELINE_STATE_FLAGS*>(const_cast<void*>(value->pDesc)));
+            break;
+        case D3D12_STATE_SUBOBJECT_TYPE_DEPTH_STENCIL1:
+            wrapper->depth_stencil_desc1 = DecodeAllocator::Allocate<StructPointerDecoder<Decoded_D3D12_DEPTH_STENCIL_DESC1>>();
+            bytes_read += wrapper->depth_stencil_desc1->Decode(buffer2, buffer_size2);
+            value->pDesc = wrapper->depth_stencil_desc1->GetPointer();
+            break;
+        case D3D12_STATE_SUBOBJECT_TYPE_VIEW_INSTANCING:
+            wrapper->view_instancing_desc = DecodeAllocator::Allocate<StructPointerDecoder<Decoded_D3D12_VIEW_INSTANCING_DESC>>();
+            bytes_read += wrapper->view_instancing_desc->Decode(buffer2, buffer_size2);
+            value->pDesc = wrapper->view_instancing_desc->GetPointer();
+            break;
+        case D3D12_STATE_SUBOBJECT_TYPE_GENERIC_PROGRAM:
+            wrapper->generic_program_desc = DecodeAllocator::Allocate<StructPointerDecoder<Decoded_D3D12_GENERIC_PROGRAM_DESC>>();
+            bytes_read += wrapper->generic_program_desc->Decode(buffer2, buffer_size2);
+            value->pDesc = wrapper->generic_program_desc->GetPointer();
+            break;
+        case D3D12_STATE_SUBOBJECT_TYPE_DEPTH_STENCIL2:
+            wrapper->depth_stencil_desc2 = DecodeAllocator::Allocate<StructPointerDecoder<Decoded_D3D12_DEPTH_STENCIL_DESC2>>();
+            bytes_read += wrapper->depth_stencil_desc2->Decode(buffer2, buffer_size2);
+            value->pDesc = wrapper->depth_stencil_desc2->GetPointer();
+            break;
         case D3D12_STATE_SUBOBJECT_TYPE_MAX_VALID:
             break;
     }
@@ -1147,6 +1243,28 @@ size_t DecodeStruct(const uint8_t* buffer, size_t buffer_size, Decoded_D3D12_SUB
     return bytes_read;
 }
 
+size_t DecodeStruct(const uint8_t* buffer, size_t buffer_size, Decoded_D3D12_GENERIC_PROGRAM_DESC* wrapper)
+{
+    assert((wrapper != nullptr) && (wrapper->decoded_value != nullptr));
+
+    size_t                      bytes_read = 0;
+    D3D12_GENERIC_PROGRAM_DESC* value      = wrapper->decoded_value;
+
+    bytes_read += wrapper->ProgramName.Decode((buffer + bytes_read), (buffer_size - bytes_read));
+    value->ProgramName = wrapper->ProgramName.GetPointer();
+    bytes_read +=
+        ValueDecoder::DecodeUInt32Value((buffer + bytes_read), (buffer_size - bytes_read), &(value->NumExports));
+    bytes_read += wrapper->pExports.Decode((buffer + bytes_read), (buffer_size - bytes_read));
+    value->pExports = const_cast<LPCWSTR*>(wrapper->pExports.GetPointer());
+    bytes_read +=
+        ValueDecoder::DecodeUInt32Value((buffer + bytes_read), (buffer_size - bytes_read), &(value->NumSubobjects));
+    wrapper->ppSubobjects = DecodeAllocator::Allocate<StructPointerDecoder<Decoded_D3D12_STATE_SUBOBJECT*>>();
+    bytes_read += wrapper->ppSubobjects->Decode((buffer + bytes_read), (buffer_size - bytes_read));
+    value->ppSubobjects = wrapper->ppSubobjects->GetPointer();
+
+    return bytes_read;
+}
+
 size_t DecodeStruct(const uint8_t* buffer, size_t buffer_size, Decoded_D3D12_BARRIER_GROUP* wrapper)
 {
     assert((wrapper != nullptr) && (wrapper->decoded_value != nullptr));
@@ -1207,5 +1325,138 @@ size_t DecodeStruct(const uint8_t* buffer, size_t buffer_size, Decoded_D3D12_SAM
     return bytes_read;
 }
 
+size_t DecodeStruct(const uint8_t* buffer, size_t buffer_size, Decoded_D3D12_SHADER_NODE* wrapper)
+{
+    size_t             bytes_read = 0;
+    WStringDecoder     Shader;
+    D3D12_SHADER_NODE* value = wrapper->decoded_value;
+
+    bytes_read += Shader.Decode((buffer + bytes_read), (buffer_size - bytes_read));
+    bytes_read +=
+        ValueDecoder::DecodeEnumValue((buffer + bytes_read), (buffer_size - bytes_read), &(value->OverridesType));
+
+    switch (value->OverridesType)
+    {
+        case D3D12_NODE_OVERRIDES_TYPE_BROADCASTING_LAUNCH:
+            wrapper->broadcasting_launch_overrides =
+                DecodeAllocator::Allocate<StructPointerDecoder<Decoded_D3D12_BROADCASTING_LAUNCH_OVERRIDES>>();
+            bytes_read +=
+                wrapper->broadcasting_launch_overrides->Decode((buffer + bytes_read), (buffer_size - bytes_read));
+            value->pBroadcastingLaunchOverrides = wrapper->broadcasting_launch_overrides->GetPointer();
+            break;
+        case D3D12_NODE_OVERRIDES_TYPE_COALESCING_LAUNCH:
+            wrapper->coalescing_launch_overrides =
+                DecodeAllocator::Allocate<StructPointerDecoder<Decoded_D3D12_COALESCING_LAUNCH_OVERRIDES>>();
+            bytes_read +=
+                wrapper->coalescing_launch_overrides->Decode((buffer + bytes_read), (buffer_size - bytes_read));
+            value->pCoalescingLaunchOverrides = wrapper->coalescing_launch_overrides->GetPointer();
+            break;
+        case D3D12_NODE_OVERRIDES_TYPE_THREAD_LAUNCH:
+            wrapper->thread_launch_overrides =
+                DecodeAllocator::Allocate<StructPointerDecoder<Decoded_D3D12_THREAD_LAUNCH_OVERRIDES>>();
+            bytes_read += wrapper->thread_launch_overrides->Decode((buffer + bytes_read), (buffer_size - bytes_read));
+            value->pThreadLaunchOverrides = wrapper->thread_launch_overrides->GetPointer();
+            break;
+        case D3D12_NODE_OVERRIDES_TYPE_COMMON_COMPUTE:
+            wrapper->common_compute_node_overrides =
+                DecodeAllocator::Allocate<StructPointerDecoder<Decoded_D3D12_COMMON_COMPUTE_NODE_OVERRIDES>>();
+            bytes_read +=
+                wrapper->common_compute_node_overrides->Decode((buffer + bytes_read), (buffer_size - bytes_read));
+            value->pCommonComputeNodeOverrides = wrapper->common_compute_node_overrides->GetPointer();
+            break;
+        default:
+            break;
+    }
+
+    return bytes_read;
+}
+
+size_t DecodeStruct(const uint8_t* buffer, size_t buffer_size, Decoded_D3D12_NODE* wrapper)
+{
+    size_t      bytes_read = 0;
+    D3D12_NODE* value      = wrapper->decoded_value;
+
+    bytes_read += ValueDecoder::DecodeEnumValue((buffer + bytes_read), (buffer_size - bytes_read), &(value->NodeType));
+
+    switch (value->NodeType)
+    {
+        case D3D12_NODE_TYPE_SHADER:
+            wrapper->shader                = DecodeAllocator::Allocate<Decoded_D3D12_SHADER_NODE>();
+            wrapper->shader->decoded_value = &(value->Shader);
+            bytes_read += DecodeStruct((buffer + bytes_read), (buffer_size - bytes_read), wrapper->shader);
+            break;
+        default:
+            break;
+    }
+
+    return bytes_read;
+}
+
+size_t DecodeStruct(const uint8_t* buffer, size_t buffer_size, Decoded_D3D12_SET_PROGRAM_DESC* wrapper)
+{
+    size_t                  bytes_read = 0;
+    D3D12_SET_PROGRAM_DESC* value      = wrapper->decoded_value;
+
+    bytes_read += ValueDecoder::DecodeEnumValue((buffer + bytes_read), (buffer_size - bytes_read), &(value->Type));
+
+    switch (value->Type)
+    {
+        case D3D12_PROGRAM_TYPE_GENERIC_PIPELINE:
+            wrapper->generic_pipeline = DecodeAllocator::Allocate<Decoded_D3D12_SET_GENERIC_PIPELINE_DESC>();
+            wrapper->generic_pipeline->decoded_value = &(value->GenericPipeline);
+            bytes_read += DecodeStruct((buffer + bytes_read), (buffer_size - bytes_read), wrapper->generic_pipeline);
+            break;
+        case D3D12_PROGRAM_TYPE_RAYTRACING_PIPELINE:
+            wrapper->raytracing_pipeline = DecodeAllocator::Allocate<Decoded_D3D12_SET_RAYTRACING_PIPELINE_DESC>();
+            wrapper->raytracing_pipeline->decoded_value = &(value->RaytracingPipeline);
+            bytes_read += DecodeStruct((buffer + bytes_read), (buffer_size - bytes_read), wrapper->raytracing_pipeline);
+            break;
+        case D3D12_PROGRAM_TYPE_WORK_GRAPH:
+            wrapper->work_graph                = DecodeAllocator::Allocate<Decoded_D3D12_SET_WORK_GRAPH_DESC>();
+            wrapper->work_graph->decoded_value = &(value->WorkGraph);
+            bytes_read += DecodeStruct((buffer + bytes_read), (buffer_size - bytes_read), wrapper->work_graph);
+            break;
+        default:
+            break;
+    }
+
+    return bytes_read;
+}
+
+size_t DecodeStruct(const uint8_t* buffer, size_t buffer_size, Decoded_D3D12_DISPATCH_GRAPH_DESC* wrapper)
+{
+    size_t                     bytes_read = 0;
+    D3D12_DISPATCH_GRAPH_DESC* value      = wrapper->decoded_value;
+
+    bytes_read += ValueDecoder::DecodeEnumValue((buffer + bytes_read), (buffer_size - bytes_read), &(value->Mode));
+
+    switch (value->Mode)
+    {
+        case D3D12_DISPATCH_MODE_NODE_CPU_INPUT:
+            wrapper->node_cpu_input                = DecodeAllocator::Allocate<Decoded_D3D12_NODE_CPU_INPUT>();
+            wrapper->node_cpu_input->decoded_value = &(value->NodeCPUInput);
+            bytes_read += DecodeStruct((buffer + bytes_read), (buffer_size - bytes_read), wrapper->node_cpu_input);
+            break;
+        case D3D12_DISPATCH_MODE_NODE_GPU_INPUT:
+            bytes_read += ValueDecoder::DecodeUInt64Value(
+                (buffer + bytes_read), (buffer_size - bytes_read), &(value->NodeGPUInput));
+            break;
+        case D3D12_DISPATCH_MODE_MULTI_NODE_CPU_INPUT:
+            wrapper->multi_node_cpu_input = DecodeAllocator::Allocate<Decoded_D3D12_MULTI_NODE_CPU_INPUT>();
+            wrapper->multi_node_cpu_input->decoded_value = &(value->MultiNodeCPUInput);
+            bytes_read +=
+                DecodeStruct((buffer + bytes_read), (buffer_size - bytes_read), wrapper->multi_node_cpu_input);
+            break;
+        case D3D12_DISPATCH_MODE_MULTI_NODE_GPU_INPUT:
+            bytes_read += ValueDecoder::DecodeUInt64Value(
+                (buffer + bytes_read), (buffer_size - bytes_read), &(value->MultiNodeGPUInput));
+            break;
+        default:
+            break;
+    }
+
+    return bytes_read;
+}
+
 GFXRECON_END_NAMESPACE(decode)
 GFXRECON_END_NAMESPACE(gfxrecon)
