@@ -25,6 +25,7 @@
 #include "decode/vulkan_resource_allocator.h"
 #include "decode/decoder_util.h"
 #include "decode/mark_injected_commands.h"
+#include "graphics/vulkan_resources_util.h"
 #include "vulkan/vulkan_core.h"
 #include <array>
 
@@ -49,7 +50,7 @@ VkResult VulkanVirtualSwapchain::CreateSwapchainKHR(VkResult
                                                     const VkSwapchainCreateInfoKHR*       create_info,
                                                     const VkAllocationCallbacks*          allocator,
                                                     HandlePointerDecoder<VkSwapchainKHR>* swapchain,
-                                                    const encode::VulkanDeviceTable*      device_table)
+                                                    const graphics::VulkanDeviceTable*    device_table)
 {
     VkDevice                 device = VK_NULL_HANDLE;
     VkSurfaceCapabilitiesKHR surfCapabilities{};
@@ -520,7 +521,7 @@ VkResult VulkanVirtualSwapchain::CreateSwapchainResourceData(const VulkanDeviceI
                 VK_QUEUE_FAMILY_IGNORED,                // dstQueueFamilyIndex
                 VK_NULL_HANDLE,                         // image
                 VkImageSubresourceRange{
-                    VK_IMAGE_ASPECT_COLOR_BIT,
+                    graphics::GetFormatAspectMask(swapchain_info->format),
                     0,
                     image_create_info.mipLevels,
                     0,
@@ -808,6 +809,13 @@ VkResult VulkanVirtualSwapchain::QueuePresentKHR(VkResult
         uint32_t    capture_image_index = capture_image_indices[i];
         uint32_t    replay_image_index  = present_info->pImageIndices[i];
 
+        auto aspect_mask       = graphics::GetFormatAspectMask(swapchain_info->format);
+        subresource.aspectMask = aspect_mask;
+        initial_barrier_virtual_image.subresourceRange.aspectMask   = aspect_mask;
+        final_barrier_virtual_image.subresourceRange.aspectMask     = aspect_mask;
+        initial_barrier_swapchain_image.subresourceRange.aspectMask = aspect_mask;
+        final_barrier_swapchain_image.subresourceRange.aspectMask   = aspect_mask;
+
         // Get the per swapchain resource data so we have access to the virtual swapchain-specific information.
         if (swapchain_resources_.find(swapchain_info->handle) == swapchain_resources_.end())
         {
