@@ -1,5 +1,6 @@
 /*
 ** Copyright (c) 2021 LunarG, Inc.
+** Copyright (c) 2025 Advanced Micro Devices, Inc. All rights reserved.
 **
 ** Permission is hereby granted, free of charge, to any person obtaining a
 ** copy of this software and associated documentation files (the "Software"),
@@ -412,7 +413,7 @@ HRESULT Dx12ResourceDataUtil::ReadFromResource(ID3D12Resource*
     // If the resource can be mapped, map it, copy the data, and return success.
     if (try_map_and_copy && IsResourceCpuAccessible(target_resource, kCopyTypeRead))
     {
-        data.resize(static_cast<size_t>(required_data_size));
+        data.resize(static_cast<size_t>(required_data_size), 0);
         if (CopyMappableResource(
                 target_resource, kCopyTypeRead, &data, nullptr, subresource_offsets, subresource_sizes))
         {
@@ -448,7 +449,7 @@ HRESULT Dx12ResourceDataUtil::ReadFromResource(ID3D12Resource*
     // After the command list has completed, map the copy resource and read its data.
     if (!batching && SUCCEEDED(result))
     {
-        data.resize(static_cast<size_t>(required_data_size));
+        data.resize(static_cast<size_t>(required_data_size), 0);
         result = MapSubresourceAndReadData(staging_resource, 0, static_cast<size_t>(required_data_size), data.data());
     }
 
@@ -526,13 +527,13 @@ HRESULT Dx12ResourceDataUtil::WriteToResource(ID3D12Resource*
             }
             else if (layout_sizes[i] < subresource_sizes[i])
             {
-                GFXRECON_LOG_ERROR("The size of the data to be copied to the subresource is greater than the size of the subresource's copyable footprint."
-                                     "Not all data can be written to the subresource. (data size = %" PRIu64
-                                     ", footprint size = %" PRIu64 ", subresouce index = %" PRIu32 ").",
-                                     subresource_sizes[i],
-                                     layout_sizes[i],
-                                     i);
-
+                GFXRECON_LOG_ERROR("The size of the data to be copied to the subresource is greater than the size of "
+                                   "the subresource's copyable footprint."
+                                   "Not all data can be written to the subresource. (data size = %" PRIu64
+                                   ", footprint size = %" PRIu64 ", subresouce index = %" PRIu32 ").",
+                                   subresource_sizes[i],
+                                   layout_sizes[i],
+                                   i);
             }
             size_t layout_offset = static_cast<size_t>(layout_offsets[i]);
             size_t layout_size   = static_cast<size_t>(layout_sizes[i]);
@@ -839,6 +840,14 @@ Dx12ResourceDataUtil::ExecuteCopyCommandList(ID3D12Resource*
     return result;
 }
 
+void Dx12ResourceDataUtil::InitializeMetaCommand(ID3D12MetaCommand* pMetaCommand,
+                                                 const void*        pInitializationParametersData,
+                                                 SIZE_T             InitializationParametersDataSizeInBytes)
+{
+    command_list_->InitializeMetaCommand(
+        pMetaCommand, pInitializationParametersData, InitializationParametersDataSizeInBytes);
+}
+
 HRESULT
 Dx12ResourceDataUtil::ExecuteTransitionCommandList(ID3D12Resource*                             target_resource,
                                                    const std::vector<dx12::ResourceStateInfo>& before_states,
